use anyhow::{bail, Result};
use kdash_protocol::Orientation;
use url::Url;

use crate::battery::BatteryStatus;

#[derive(Debug)]
pub struct Api {
    pub jwt: String,
    pub config_url: Url,
    pub image_url: Url,
}

impl Api {
    pub fn new(jwt: String, url: Url) -> Result<Self> {
        Ok(Self {
            jwt,
            config_url: url.join("config")?,
            image_url: url.join("image")?,
        })
    }

    fn authorization_bearer(bearer: &str) -> String {
        format!("Bearer {}", bearer)
    }

    pub async fn fetch_config(&self) -> Result<kdash_protocol::Config> {
        let config = reqwest::Client::new()
            .get(self.config_url.to_owned())
            .header(
                reqwest::header::AUTHORIZATION,
                Self::authorization_bearer(&self.jwt),
            )
            .send()
            .await?
            .json::<kdash_protocol::Config>()
            .await?;

        Ok(config)
    }

    pub async fn fetch_config_with_post_device_state(
        &self,
        battery: &BatteryStatus,
    ) -> Result<kdash_protocol::Config> {
        let config = reqwest::Client::new()
            .post(self.config_url.to_owned())
            .header(
                reqwest::header::AUTHORIZATION,
                Self::authorization_bearer(&self.jwt),
            )
            .json(&kdash_protocol::DeviceStatePost {
                battery_charging: battery.is_charging,
                battery_level: kdash_protocol::Percent::try_new(battery.percentage)?,
                battery_current: battery.current,
                battery_voltage: battery.voltage,
            })
            .send()
            .await?
            .json::<kdash_protocol::Config>()
            .await?;

        Ok(config)
    }

    pub async fn fetch_image<'a>(
        &self,
        size: (u32, u32),
        orientation: Orientation,
    ) -> Result<image::ImageBuffer<image::Luma<u8>, Vec<u8>>> {
        let response = reqwest::Client::new()
            .get(self.image_url.to_owned())
            .header(
                reqwest::header::AUTHORIZATION,
                Self::authorization_bearer(&self.jwt),
            )
            .send()
            .await?;

        let content_type = match response.headers().get(reqwest::header::CONTENT_TYPE) {
            Some(value) => value.to_str()?,
            None => bail!("Image response returned no Content-Type header"),
        };

        let image_format = match image::ImageFormat::from_mime_type(content_type) {
            Some(x) => x,
            None => bail!("Invalid image MIME type: {}", content_type),
        };

        if image_format != image::ImageFormat::Png && image_format != image::ImageFormat::Bmp {
            bail!("Invalid image MIME type: {}", image_format.to_mime_type())
        }

        let data = response.bytes().await?;
        let reader = image::ImageReader::with_format(std::io::Cursor::new(data), image_format);

        let mut img = reader.decode()?.crop_imm(0, 0, size.0, size.1);
        match orientation {
            Orientation::PortraitUp | Orientation::PortraitDown => {}
            Orientation::LandscapeLeft | Orientation::LandscapeRight => {
                img = img.rotate270();
            }
        }

        Ok(img.into_luma8())
    }
}
