use anyhow::Result;
use std::fs;

use crate::{parse, utils};

#[derive(Debug)]
pub struct BatteryStatus {
    pub is_charging: bool,
    pub percentage: u8,
    pub current: i16,
    pub voltage: i16,
}

pub fn get_battery_status() -> Result<BatteryStatus> {
    log::info!("Getting battery status");

    let charging_file = utils::kdb_get("system/driver/charger/SYS_CHARGING_FILE")?;
    let is_charging_str = fs::read_to_string(charging_file)?;
    let is_charging = parse::parse_number_from_start_unsigned::<u8>(&is_charging_str)? == 1;

    let charge_str = utils::exec_command("gasgauge-info", &["-c"])?;
    let charge = parse::parse_number_from_start_unsigned::<u8>(&charge_str)?;

    let charge_current_str = utils::exec_command("gasgauge-info", &["-l"])?;
    let charge_current = parse::parse_number_from_start_signed::<i16>(&charge_current_str)?;

    let voltage_str = utils::exec_command("gasgauge-info", &["-v"])?;
    let voltage = parse::parse_number_from_start_signed::<i16>(&voltage_str)?;

    let battery = BatteryStatus {
        is_charging,
        percentage: charge,
        current: charge_current,
        voltage,
    };

    log::info!("Got battery status: {:?}", battery);

    Ok(battery)
}

pub fn restart_powerd() -> Result<()> {
    log::info!("Restarting powerd");
    utils::exec_command_discard("/etc/init.d/powerd", &["restart"])
}

pub fn restart_powerd_config_condition(
    battery: &BatteryStatus,
    battery_config: &kdash_protocol::BatteryConfig,
) -> Result<()> {
    if battery.is_charging
        && battery.percentage <= battery_config.restart_powerd_threshold.into_inner()
        && battery.current <= 0
    {
        log::info!(
            "Battery charge below threshold ({} <= {}): restarting powerd",
            battery.percentage,
            battery_config.restart_powerd_threshold.into_inner()
        );
        restart_powerd()
    } else {
        Ok(())
    }
}
