use anyhow::Result;
use chrono::prelude::*;
use embedded_graphics::{image::Image, prelude::*};
use embedded_layout::prelude::*;
use std::{thread, time::Duration};

pub mod api;
pub mod app;
pub mod assets;
pub mod battery;
pub mod config;
pub mod fb;
mod parse;
pub mod rtc;
pub mod utils;
pub mod wifi;

pub async fn run_once(
    state: &app::State,
    display: &mut fb::FramebufferDisplay,
) -> Result<Option<kdash_protocol::Config>> {
    let config = state.config.lock().await;

    utils::set_cpu_powersaving()?;
    utils::prevent_screensaver(&state.lipc.lock().unwrap())?;

    let battery = battery::get_battery_status()?;
    battery::restart_powerd_config_condition(&battery, &config.battery)?;
    if battery.percentage <= config.battery.low.into_inner() {
        log::info!(
            "Battery low: {}% <= {}%",
            battery.percentage,
            config.battery.low.into_inner()
        );

        Image::new(&*assets::ERROR_BATTERY_LOW_IMAGE, Point::zero())
            .align_to(&*display, horizontal::Center, vertical::Center)
            .draw(display)?;

        fb::eips_clear()?;
        display.flush_full_update()?;
        thread::sleep(Duration::from_secs(1));

        rtc::sleep(config.time.delay_low_battery.to_std()?.as_secs())?;
        thread::sleep(Duration::from_secs(30));
    }

    log::info!("Enabling and checking WiFi");
    wifi::set_wifid_enable(&state.lipc.lock().unwrap(), true)?;
    let wlan_connected =
        wifi::check_connection(&state.lipc.lock().unwrap(), &state.app_config.net)?;
    wifi::log_connection(&state.lipc.lock().unwrap(), &state.app_config.net)?;

    let mut new_config: Option<kdash_protocol::Config> = None;

    if wlan_connected {
        log::info!("Connected to WiFi");
        let gateway = wifi::get_gateway(&state.app_config.net)?;
        log::info!("Found default gateway: {:?}", gateway);

        if gateway.is_none() {
            log::info!("Default gateway lost after sleep");
            log::info!("Setting default gateway to: {}", state.app_config.router_ip);
            wifi::route_add_default_gateway_router_ip(state.app_config.router_ip)?;
        }

        log::info!("Fetching config");

        let fetch_succ = match state
            .api
            .fetch_config_with_post_device_state(&battery)
            .await
        {
            Ok(value) => {
                new_config = Some(value);

                log::info!("Downloading and drawing image");

                match state
                    .api
                    .fetch_image(
                        (
                            display.fb.var_screen_info.xres_virtual,
                            display.fb.var_screen_info.yres_virtual,
                        ),
                        display.fb_orientation.orientation,
                    )
                    .await
                {
                    Ok(buf) => {
                        let raw = fb::image_buf_to_raw(&buf);
                        Image::new(&raw, Point::zero()).draw(display)?;
                        true
                    }
                    Err(e) => {
                        log::error!("Downloaded image with error: {}", e);
                        false
                    }
                }
            }
            Err(e) => {
                dbg!(&e);
                log::error!("Fetched config with error: {}", e);
                false
            }
        };

        if !fetch_succ {
            Image::new(&*assets::ERROR_NETWORK_IMAGE, Point::zero())
                .align_to(&*display, horizontal::Center, vertical::Center)
                .draw(display)?;
        }

        if battery.percentage <= config.battery.alert.into_inner() {
            let now_str = Utc::now().to_rfc3339_opts(SecondsFormat::Secs, true);
            let text = format!("Battery at {}%, please charge!", battery.percentage);
            let status_box = fb::widgets::status_box::StatusBox::new_with_default_style(
                Point::zero(),
                &now_str,
                &text,
            );
            let mut padding = fb::widgets::padding::Paddding::new(
                Point::zero(),
                fb::widgets::status_box::DEFAULT_PADDING,
                None,
                status_box,
            );

            match config.device.status_box_location {
                kdash_protocol::Corner::TopLeft => {}
                kdash_protocol::Corner::TopRight => {
                    padding.align_to_mut(display, horizontal::Right, vertical::Top);
                }
                kdash_protocol::Corner::BottomLeft => {
                    padding.align_to_mut(display, horizontal::Left, vertical::Bottom);
                }
                kdash_protocol::Corner::BottomRight => {
                    padding.align_to_mut(display, horizontal::Right, vertical::Bottom);
                }
            }

            padding.draw(display)?;
        }

        if config.device.full_refresh {
            fb::eips_clear()?;
        }
        display.flush_full_update()?;
    }

    Ok(new_config)
}
