use anyhow::Result;
use openlipc_dyn::Lipc;
use std::{net, thread, time};

use crate::utils;

pub fn set_wifid_enable(lipc: &Lipc, value: bool) -> Result<()> {
    log::info!("Enabling wifi");
    lipc.set_int_prop("com.lab126.wifid", "enable", if value { 1 } else { 0 })?;
    Ok(())
}

fn get_cm_state(lipc: &Lipc) -> Result<String> {
    lipc.get_str_prop("com.lab126.wifid", "cmState")
}

fn get_connected(lipc: &Lipc) -> Result<bool> {
    Ok(get_cm_state(lipc)? == "CONNECTED")
}

fn get_signal_strength(lipc: &Lipc) -> Result<String> {
    lipc.get_str_prop("com.lab126.wifid", "signalStrength")
}

fn reconnect(net: &str) -> Result<()> {
    log::info!("Reconnecting WiFi");
    utils::exec_command_discard("/usr/bin/wpa_cli", &["-i", net, "reconnect"])?;
    Ok(())
}

pub fn log_connection(lipc: &Lipc, net: &str) -> Result<()> {
    let ifconfig = utils::exec_command("ifconfig", &[net])?;
    log::info!("ifconfig: {}", ifconfig);

    let cm_state = get_cm_state(lipc)?;
    log::info!("cmState: {}", cm_state);

    let signal_strength = get_signal_strength(lipc)?;
    log::info!("signalStrength: {}", signal_strength);

    Ok(())
}

pub fn check_connection(lipc: &Lipc, net: &str) -> Result<bool> {
    let mut wlan_connected = true;
    let mut wlan_counter: u64 = 0;

    while !get_connected(lipc)? {
        if wlan_counter > 5 {
            log::info!("Trying WiFi reconnect");
            reconnect(net)?;
        } else if wlan_counter > 30 {
            log::info!("No known WiFi found");
            log_connection(lipc, net)?;
            wlan_connected = false;
            // TODO: eips wifi error
            break;
        }

        wlan_counter += 1;
        log::info!("Waiting for WiFi: {}", wlan_counter);
        thread::sleep(time::Duration::from_secs(wlan_counter));
    }

    Ok(wlan_connected)
}

pub fn get_gateway(net: &str) -> Result<Option<net::IpAddr>> {
    let routes = utils::exec_command("ip", &["route"])?;
    for line in routes.lines() {
        if line.contains("default") && line.contains(net) {
            if let Some(gateway) = line.split_whitespace().nth(2) {
                return Ok(Some(gateway.parse()?));
            }
        }
    }

    Ok(None)
}

pub fn route_add_default_gateway_router_ip(router_ip: net::IpAddr) -> Result<()> {
    utils::exec_command_discard(
        "route",
        &["add", "default", "gw", router_ip.to_string().as_str()],
    )
}
